import os


def detect_periodic_tasks(project_root):
    tasks = {}
    file_path = os.path.join(project_root)
    for root, dirs, files in os.walk(file_path):
        for filename in files:
            if os.path.basename(root) == "tasks":
                if filename.startswith("periodic_tasks"):
                    full_path = os.path.join(root, filename)
                    ldict = locals()
                    exec(open(full_path).read(), globals(), ldict)
                    try:
                        tasks.update(ldict["periodic_tasks"])
                    except NameError:
                        pass
    return tasks



import functools

from django.contrib.postgres.indexes import GinIndex


class UpperGinIndex(GinIndex):
    def create_sql(self, model, schema_editor, using=""):
        statement = super().create_sql(model, schema_editor, using=using)
        quote_name = statement.parts["columns"].quote_name

        def upper_quoted(column):
            return f"UPPER({quote_name(column)})"

        statement.parts["columns"].quote_name = upper_quoted
        return statement


def nest_statement(item):
    output = {}
    for key, value in item.items():
        path = key.split(".")
        target = functools.reduce(lambda d, k: d.setdefault(k, {}), path[:-1], output)
        target[path[-1]] = value
    return output


def dictfetchall(cursor, nest_dict: bool = False):
    "Return all rows from a cursor as a dict"
    columns = [col[0] for col in cursor.description]
    dict_cursor = [dict(zip(columns, row)) for row in cursor.fetchall()]

    if nest_dict:
        nested_dict_cursor = []
        for item in dict_cursor:
            nested_item = nest_statement(item)
            nested_dict_cursor.append(nested_item)
        return nested_dict_cursor
    else:
        return dict_cursor



from django.core.exceptions import PermissionDenied
from rest_framework import permissions


def permission_checker(perm):
    def wrapped_decorator(func):
        def wrapped_mutation(cls, root, info, **input):
            # make sure of these arguments to the wrapped mutation
            user = info.context.user
            if isinstance(perm, str):
                perms = (perm,)
            else:
                perms = perm

            if user.has_perms(perms):
                return func(cls, root, info, **input)
            raise PermissionDenied("Permission Denied.")

        return wrapped_mutation

    return wrapped_decorator


class IsOwnerOrReadOnly(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `owner` attribute.
    """

    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True

        # Instance must have an attribute named `owner`.
        return obj.owner == request.user


class BaseModelPermissions(permissions.DjangoModelPermissions):
    def get_custom_perms(self, method, view):
        app_name = view.model._meta.app_label
        return [
            app_name + "." + perms for perms in view.extra_perms_map.get(method, [])
        ]

    def has_permission(self, request, view):
        perms = self.get_required_permissions(request.method, view.model)
        perms.extend(self.get_custom_perms(request.method, view))
        return (
            request.user
            and (request.user.is_authenticated() or not self.authenticated_users_only)
            and request.user.has_perms(perms)
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_handy_utils',
 'django_handy_utils.filters',
 'django_handy_utils.models',
 'django_handy_utils.views']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2', 'djangorestframework>=3']

extras_require = \
{'datasci': ['pandas>=1.1,<2.0', 'numpy>=1.19,<2.0']}

setup_kwargs = {
    'name': 'django-handy-utils',
    'version': '1.1.2',
    'description': 'Handy utilities for working on Django projects',
    'long_description': '# django-handy-utils\n\n`django-handy-utils` contains various model, view, filter, and other miscellaneous utilities that are standardized across projects.\n\n## Installation\n\n```shell\npip install django-handy-utils --upgrade\n```\n\n## Usage in Django\n\nTO COME\n\n\n## Contributors\n\nAll contributors are welcome.',
    'author': 'Marc Ford',
    'author_email': 'mrfxyz567@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mfdeux/django-handy-utils',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import uuid

from django.db import models
from django.utils import timezone


class UUIDMixin(models.Model):
    """
    Generate uuid as id field and act as primary key
    """

    id = models.UUIDField(
        default=uuid.uuid4,
        primary_key=True,
        editable=False,
        help_text="Unique ID of the instance",
    )

    class Meta:
        abstract = True


class TimeTrackingMixin(models.Model):
    """
    Track created at and modified at time
    """

    created_at = models.DateTimeField(
        default=timezone.now,
        null=False,
        blank=False,
        editable=False,
        help_text="Datetime the instance was created",
    )
    modified_at = models.DateTimeField(
        default=timezone.now,
        null=False,
        blank=False,
        help_text="Datetime the instance was last modified at",
    )

    # Update modified_at field upon save.
    def save(self, *args, **kwargs):
        if getattr(self, "modified_at"):
            self.modified_at = timezone.now()
        super(TimeTrackingMixin, self).save(*args, **kwargs)

    class Meta:
        abstract = True


class SoftDeleteMixin(models.Model):
    """
    Allow model to be soft deleted and track deleted time
    """

    is_deleted = models.BooleanField(
        default=False, help_text="Whether the instance is soft deleted or not"
    )
    deleted_at = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Datetime the instance was soft deleted at (if applicable)",
    )

    def soft_delete(self):
        self.is_deleted = True
        self.deleted_at = timezone.now()
        super(SoftDeleteMixin, self).save()

    class Meta:
        abstract = True


class DisablableMixin(models.Model):
    """
    Allow model to be disabled and track when disabled
    """

    is_enabled = models.BooleanField(
        default=True, help_text="Whether the instance is enabled or not"
    )
    disabled_at = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Datetime the instance was disabled at (if applicable)",
    )

    class Meta:
        abstract = True


class HideableMixin(models.Model):
    """
    Allow model to be hidden
    """

    is_hidden = models.BooleanField(
        default=False, help_text="Whether the instance is hidden or not"
    )
    hidden_at = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Datetime the instance was hidden at (if applicable)",
    )

    class Meta:
        abstract = True
